/* GP3EZWEB by AWC - http://www.awce.com */

#include <stdio.h>
#include <string.h>

// format of the GP3EZ output lines (conservatively spaced)
typedef struct 
{
  char timedate[64];
  char value[64];
  char id[64];
} record;

// last state
char state[64];
// Maximum #of different values
record records[128];
// high water mark in array
int recordmax=0;


int main(int argc, char *argv[])
{
  FILE *in=stdin;
  FILE *out=stdout;
  FILE *template;
  char line[1025];
  char lineout[1025];
  char *token;
  // set up file handles
  if (argc<=1)
    {
      fprintf(stderr,"Usage: gp3ezweb template [in|-] [out|-]\nUse dashes for standard input or output\n");
      return 2;
    }
  template=fopen(argv[1],"r");
  if (!template) return 4;
  if (argc>2 && *argv[2]!='-') in=fopen(argv[2],"r");
  if (!in) { perror(argv[2]); return 1; }
  if (argc>3 && *argv[3]!='-') out=fopen(argv[3],"w");
  if (!out) { perror(argv[2]); return 3; }
  // read input -- take last value and last state only
  while (!feof(in)&&fgets(line,sizeof(line),in))
    {
      int i;
      token=strtok(line,",");
      if (!token) continue; // huh?
      // tenatively record this line
      strcpy(records[recordmax].timedate,token);
      token=strtok(NULL,",");
      if (!token) continue;
      strcpy(records[recordmax].value,token);
      token=strtok(NULL,",");
      if (!token) continue;
      strcpy(records[recordmax].id,token);
      // look to see if we can replace an old one
      for (i=0;i<recordmax;i++)
	{
	  if (!strcmp(token,records[i].id))
	    {
	      strcpy(records[i].timedate,records[recordmax].timedate);
	      strcpy(records[i].value,records[recordmax].value);
	      break;
	    }
	}
      // if we couldn't replace an old one, move the highwater mark
      if (i>=recordmax)
	{
	  recordmax++;
	}
      // get state
      token=strtok(NULL,"\n");
      strcpy(state,token);
    }
  if (in!=stdin) fclose(in);

  // now copy the template to the output
  while (!feof(template) && fgets(line,sizeof(line),template))
    {
      char *p1,*p2;
      int offset=0, i, tokenoffset=0;
      *lineout='\0';

      // find {}
      while (p1=strchr(line+offset,'{'))
	{
	  *p1='\0';
	  strcat(lineout,line+offset);
	  p2=strchr(p1+1,'}');
	  if (!p2) break;
	  offset=(p2+1)-line;
	  *p2='\0';
	  token=p1+1;
	  if (!strcmp(token,"{"))  // literal {
	    {
	      strcat(lineout,"{");
	      continue;
	    }
	  if (!strcmp(token,"$state"))  // state
	    {
	      strcat(lineout,state);
	      continue;
	    }
	  if (*token=='@') tokenoffset=1;  // time stamp
	  for (i=0;i<recordmax;i++)
	    {
	      if (!strcmp(token+tokenoffset,records[i].id))
		{
		  strcat(lineout,tokenoffset?records[i].timedate:records[i].value);
		  break;
		}
	    }
	  
	}
      strcat(lineout,line+offset);
      fputs(lineout,out);
    }
  close(template);
  if (out!=stdout) close(out);
  return 0;
}
